"""Rename client.preconsent in client.trusted.

Revision ID: 1740765703
Revises: 1738929510
Create Date: 2025-02-28 19:01:43.485715

"""

from collections.abc import Sequence

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "1740765703"
down_revision: str | None = "1738929510"
branch_labels: str | Sequence[str] | None = ()
depends_on: str | Sequence[str] | None = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###

    with op.batch_alter_table("client") as batch_op:
        batch_op.alter_column("preconsent", new_column_name="trusted")
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("client") as batch_op:
        batch_op.alter_column("trusted", new_column_name="preconsent")
    # ### end Alembic commands ###
