{# Macros for Flask flash message rendering #}
{% macro messages() %}
    {% with messages = get_flashed_messages(with_categories=true) %}
        {% for category, message in messages %}
            <div class="ui message {{ category }}">
                {{ message }}
            </div>
        {% endfor %}
    {% endwith %}
{% endmacro %}
