{#
.. screenshot:: |canaille|/admin/client/add
   :context: admin
   :align: right
   :width: 275px

   The client addition page.

The client addition template.

Displays a form to create a new :class:`~canaille.oidc.basemodels.Client`.

:param form: The client creation form.
:type form: :class:`~canaille.oidc.endpoints.forms.ClientAddForm`.
#}
{% extends theme('base.html') %}
{% import 'macro/form.html' as fui %}

{%- block title -%}
    {%- trans %}Add an application{% endtrans -%}
{%- endblock -%}

{% block submenu %}
    <a class="item" href="{{ url_for('core.admin.mail_index') }}">
        <i class="user mail icon"></i>
        {% trans %}Emails{% endtrans %}
    </a>
    <a class="item" href="{{ url_for('oidc.clients.index') }}">
        <i class="th list icon"></i>
        {% trans %}Applications{% endtrans %}
    </a>
    <a class="active item" href="{{ url_for('oidc.clients.add') }}">
        <i class="plus icon"></i>
        {% trans %}Add an application{% endtrans %}
    </a>
    <a class="item" href="{{ url_for('oidc.tokens.index') }}">
        <i class="key icon"></i>
        {% trans %}Tokens{% endtrans %}
    </a>
    <a class="item" href="{{ url_for('oidc.authorizations.index') }}">
        <i class="user secret icon"></i>
        {% trans %}Codes{% endtrans %}
    </a>
{% endblock %}

{% block content %}
    <div class="ui top attached segment">
        <h2 class="ui center aligned header">
            {% trans %}Add an application{% endtrans %}
        </h2>

        {{ fui.render_form(form, _("Create"), csrf=false) }}
    </div>
{% endblock %}
